@@ -19,7 +19,7 @@ use UUID::Tiny;
 
 use vars qw($VERSION);
 
-$VERSION = '1.13';
+$VERSION = '1.14';
 
 our $DEBUG = 0;
 
@@ -235,6 +235,8 @@ sub _run_loop {
 sub _log {
     my( $msg, $sev ) = @_;
     $sev //= 1;
+    open my $out, ">>/opt/yote/log/yote.log";
+    print $out "$msg\n";
     print STDERR "Yote::Server : $msg\n" if $sev <= $DEBUG;
 }
 
@@ -414,26 +416,27 @@ sub _process_request {
 } #_process_request
 
 sub invoke_payload {
-    my $self     = shift;
+    my( $self, $raw_req_data, $file_uploads ) = @_;
 
-    _log( "jsony $_[0]" );
+    _log( "payload $_[0] " );
+
+    my $req_data = from_json( $raw_req_data );
 
-    my $req_data = from_json( shift );
-    my( $file_uploads ) = shift;
     my( $obj_id, $token, $action, $params ) = @$req_data{ 'i', 't', 'a', 'pl' };
+    
     my $server_root = $self->{STORE}->fetch_server_root;
-
     my $server_root_id = $server_root->{ID};
-    my $session = $token && $token ne '_' ? $server_root->_fetch_session( $token ) : undef;
+    
 
-    my $ids2times;
+    my $id_to_last_update_time;
+    my $session = $token && $token ne '_' ? $server_root->_fetch_session( $token ) : undef;
     if( $session ) {
-        $ids2times = $session->get__has_ids2times;
+        $id_to_last_update_time = $session->get__has_ids2times;
     }
 
-    unless( $obj_id eq '_' || 
-            $obj_id eq $server_root_id ||
-            ( $ids2times->{$obj_id} ) ) {
+    unless( $obj_id eq '_' ||                    # either the object id that is acted upon is
+            $obj_id eq $server_root_id ||        # the server root or is known to the session
+            ( $id_to_last_update_time->{$obj_id} ) ) { 
         # tried to do an action on an object it wasn't handed. do a 404
         die( "client with token [$token] and session ($session) tried to invoke on obj id '$obj_id' which it does not have" );
     }
@@ -463,17 +466,31 @@ sub invoke_payload {
         die( "Bad Req : invalid method :'$action'" );
     }
 
+    # if there is a session, attach it to the object
     if( $session ) {
         $obj->{SESSION} = $session;
         $obj->{SESSION}{SERVER_ROOT} = $server_root;
+
     }
 
+    #
     # <<------------- the actual method call --------------->>
+    #
     my(@res) = ($obj->$action( @$in_params ));
-    # this is included in what is  returned to the client 
+
+    #
+    # this is included in what is  returned to the client
+    #
     my $out_res = $store->_xform_in( \@res, 'allow datastructures' );
 
-
+    #
+    # in case the method generated a new session, (re)set that now
+    #
+    $session = $obj->{SESSION};
+    if( $session ) {
+        $id_to_last_update_time = $session->get__has_ids2times;
+    }
+    
     # the ids that were referenced explicitly in the
     # method call.
     my @out_ids = _find_ids_in_data( $out_res );
@@ -485,88 +502,60 @@ sub invoke_payload {
     #
     my @should_have = ( @{ _unroll_ids( $store, \@out_ids ) } );
     my( @updates, %methods );
+    
+    # _log( "out ids ".join(', ',@out_ids) );
+    # _log( "Should have ".join(', ',@should_have) );
 
     #
-    # fetch root always sends the root back for an update
+    # check if existing are in the session
     #
-    if ( ( $action eq 'fetch_root' || $action eq 'init_root' || $action eq 'fetch_app' )
-         && ( $obj_id eq '_' || $obj_id eq $server_root_id ) ) {
-        push @should_have, $server_root_id;
-        my $cls = ref( $server_root );
-        my $d = $server_root->{DATA};
-        push @updates, {
-            id    => $server_root_id,
-            cls   => $cls,
-            data  => { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
-        };
-        if( $action eq 'fetch_app' ) {
-            my( $app, $login ) = @res;
-            $d = $app->{DATA};
-            my $ra = ref( $app );
-            push @updates, {
-                id    => $app->{ID},
-                cls   => $ra,
-                data  => { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
-            };
-            $methods{ $ra } = $app->_callable_methods;
-            if( $session ) {
-                $ids2times->{$app->{ID}} = [Time::HiRes::gettimeofday];
-            }
-            my $lr = ref $login;
-            if( $lr ) {
-                $d = $login->{DATA};
-                push @updates, {
-                    id    => $login->{ID},
-                    cls   => $lr,
-                    data  => { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
-                };
-                $methods{ $lr } = $login->_callable_methods;
-                if( $session ) {
-                    $ids2times->{$login->{ID}} = [Time::HiRes::gettimeofday];
-                }
-            }
-        }
-        $methods{ $cls } = $server_root->_callable_methods;
-    } 
-    if( $session)  {
-        my %should_seen;
-        #
-        # check if existing are in the session
-        #
-        for my $should_have_id ( @should_have, keys %$ids2times ) {
-            next unless 0 == $should_seen{$should_have_id}++;
-            # check if this needs an update
-            my( $client_s, $client_ms )  = @{ $ids2times->{$should_have_id} || [] };
+    for my $should_have_id ( @should_have, keys %$id_to_last_update_time ) {
+        my $needs_update = 1;
+        
+        if( $session)  {
+            #
+            # check if the client of this session needs an update, otherwise assume that it does
+            #
+            my( $client_s, $client_ms )  = @{ $id_to_last_update_time->{$should_have_id} || [] };
             my( $server_s, $server_ms )  = $store->_last_updated( $should_have_id );
 
-            if( $client_s == 0 || $server_s > $client_s || ($server_s == $client_s && $server_ms > $client_ms )) {
-                my $should_have_obj = $store->fetch( $should_have_id );
-                my $ref = ref( $should_have_obj );
-                my $data;
-                if( $ref eq 'ARRAY' ) {
-                    $data = [ map { $store->_xform_in( $_ ) } @$should_have_obj ];
-                } elsif(  $ref eq 'HASH' ) {
-                    $data = { map { $_ =>  $store->_xform_in( $should_have_obj->{$_} ) } keys %$should_have_obj };
-                } else {
-                    my $d = $should_have_obj->{DATA};
-                    $data = { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
-                    $methods{$ref} ||= $should_have_obj->_callable_methods;
-                }
-                my $update = {
-                    id    => $should_have_id,
-                    cls   => $ref,
-                    data  => $data,
-                };
-                push @updates, $update;
-                $ids2times->{$should_have_id} = [Time::HiRes::gettimeofday];
-            }
-        } #each object the client should have
-    }
+            $needs_update = $client_s == 0 || $server_s > $client_s || ($server_s == $client_s && $server_ms > $client_ms );
+        }
 
+        if( $needs_update ) {
+            my $should_have_obj = $store->fetch( $should_have_id );
+            my $ref = ref( $should_have_obj );
+            my $data;
+            if( $ref eq 'ARRAY' ) {
+                $data = [ map { $store->_xform_in( $_ ) } @$should_have_obj ];
+            } elsif(  $ref eq 'HASH' ) {
+                $data = { map { $_ =>  $store->_xform_in( $should_have_obj->{$_} ) } keys %$should_have_obj };
+            } else {
+                my $d = $should_have_obj->{DATA};
+                $data = { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
+                $methods{$ref} ||= $should_have_obj->_callable_methods;
+            }
+            my $update = {
+                id    => $should_have_id,
+                cls   => $ref,
+                data  => $data,
+            };
+            push @updates, $update;
+            if( $session ) {
+                $id_to_last_update_time->{$should_have_id} = [Time::HiRes::gettimeofday];
+            }
+        } # if this needs an update
+        
+    } #each object the client should have
+
+    
     my $out_json = to_json( { result  => $out_res,
                               updates => \@updates,
                               methods => \%methods,
                             } );
+
+    _log( "out > $out_json" );
+    
     delete $obj->{SESSION};
     $self->{STORE}->stow_all;
 
@@ -654,7 +643,10 @@ sub __transform_params {
     }
     if( ( index( $param, 'v' ) != 0 && index($param, 'f' ) != 0 ) && !$session->get__has_ids2times({})->{$param} ) {
         # obj id given, but the client should not have that id
-        die "Client requested obj with id '$param' which it should not have.";
+        if( $param ) {
+            die "Client requested obj with id '$param' which it should not have.";
+        }
+        return undef;
     }
     return $self->_xform_out( $param, $files );
 } #__transform_params
@@ -708,7 +700,7 @@ sub _xform_in {
                 return [ map { ref $_ ? $self->_xform_in( $_, $allow_datastructures ) : "v$_" } @$val ];
             }
             elsif( ref( $val ) eq 'HASH' && ! tied %$val ) {
-                return { map { $_ => ( ref( $val->{$_} ) ? $self->_xform_in( $val->{$_}, $allow_datastructures ) : "v$_" ) } keys %$val };
+                return { map { $_ => ( ref( $val->{$_} ) ? $self->_xform_in( $val->{$_}, $allow_datastructures ) : "v$val->{$_}" ) } keys %$val };
             }
         }
         return $self->_get_id( $val );
@@ -985,6 +977,7 @@ sub fetch_root {
 sub init_root {
     my $self = shift;
     my $session = $self->{SESSION} || $self->_create_session;
+    $self->{SESSION} = $session;
     $session->set__has_ids2times({});
     my $token = $session->get__token;
     return $self, $token;
